/* 
 * File:   AMS program
 * Author: Eric Fernandez
 *
 * Created on November 26, 2018, 5:19 PM
 */

#include <xc.h>
#include "configbits-16ex8.h"
#include "SDlib16.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
/*
 * 
 */
int pec15Table[256];
int CRC15_POLY = 0x4599;

void init_PEC15_Table()
{
int i =0, bit =0;
int remainder;
for (i = 0; i < 256; i++)
{
	remainder = i << 7;
	for (bit = 8; bit > 0; --bit)
	{
		if (remainder & 0x4000)
		{
			remainder = ((remainder << 1));
			remainder = (remainder ^ CRC15_POLY);
		}
		else{
			remainder = ((remainder << 1));
		}
	}
	pec15Table[i] = remainder&0xFFFF;
}
}

unsigned int pec15 (char *data , int len)
{
int remainder,address,i;
remainder = 16;//PEC seed
	for (i = 0; i < len; i++){
	address = ((remainder >> 7) ^ data[i]) & 0xFF;//calculate PEC table address
	remainder = (remainder << 8 ) ^ pec15Table[address];
	}
return (remainder*2);
//The CRC15 has a 0 in the LSB so the final value must be multiplied by 2
}

void initSPI(void);
short WriteReadSPI(unsigned short i);
void WakeUpAMS(void);
void killTime(void);
void init_PEC15_Table();
short ReadSPI(unsigned short i);
void checkVolts(void);
void ReadPWMRegister(void);



#define CS LATDbits.LATD7
#define LED LATDbits.LATD1

/* DONE FOR 4 BOARDS ///////////////////////////////////////////////////////
 * :: NEED TO ADJUST OV Value //////////////////////////////////////////////
 * :: MIGHT NEED TO ADJUST PEC INDIVIDUALLY *///////////////////////////////
int currentTemp=0, savedTemp;
float endSum;

void writeOV(void){
// Read Config A
    CS=0;
    //Write command
	WriteReadSPI(0x00);
	WriteReadSPI(0x02);
    // Write PEC
	char  x[2] = {0x00,0x02};
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    // Get Config A values
	unsigned short x1=0, x2=0, x3=0, x4=0, x5=0, x6=0, xPEC1 =0, xPEC0 = 0;
    unsigned short x1b=0, x2b=0, x3b=0, x4b=0, x5b=0, x6b=0, xPEC1b =0, xPEC0b = 0;
	unsigned short x1c=0, x2c=0, x3c=0, x4c=0, x5c=0, x6c=0, xPEC1c =0, xPEC0c = 0;
    unsigned short x1d=0, x2d=0, x3d=0, x4d=0, x5d=0, x6d=0, xPEC1d =0, xPEC0d = 0;
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1b = ReadSPI(0x00);
	x2b = ReadSPI(0x00);
	x3b = ReadSPI(0x00);
    x4b = ReadSPI(0x00);
	x5b = ReadSPI(0x00);
	x6b = ReadSPI(0x00);
	xPEC1b = ReadSPI(0x00);
	xPEC0b = ReadSPI(0x00);
    x1c = ReadSPI(0x00);
	x2c = ReadSPI(0x00);
	x3c = ReadSPI(0x00);
	x4c = ReadSPI(0x00);
	x5c = ReadSPI(0x00);
	x6c = ReadSPI(0x00);
	xPEC1c = ReadSPI(0x00);
	xPEC0c = ReadSPI(0x00);
    x1d = ReadSPI(0x00);
	x2d = ReadSPI(0x00);
	x3d = ReadSPI(0x00);
    x4d = ReadSPI(0x00);
	x5d = ReadSPI(0x00);
	x6d = ReadSPI(0x00);
	xPEC1d = ReadSPI(0x00);
	xPEC0d = ReadSPI(0x00);
	CS=1;
// Write to Config A
    //killTime();
    unsigned short thresholdHigh=0x4E, thresholdLow=0x20; // ((High << 8)+Low)*16*100uV = VOV // 4E2 yields 2V
    CS=0;
    //Write command
	WriteReadSPI(0x00);
	WriteReadSPI(0x01);
    // Write PEC
        char z[2] = {0x00,0x01};
        u = pec15(z,2);	
        highbyte = (u >> 8);
        lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Write config A vals & overvoltage
	ReadSPI(x1);
	ReadSPI(0x00);
	ReadSPI(thresholdLow);
	ReadSPI(thresholdHigh);
	ReadSPI(x5);
	ReadSPI(x6);
        // Calculate PEC for write
        char  y[6] = {x1, 0x00, 0x20, 0x4E, x5, x6};
        u = pec15(y,6);	
        highbyte = (u >> 8);
        lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    //
    
    ReadSPI(x1b);
	ReadSPI(0x00);
	ReadSPI(thresholdLow);
	ReadSPI(thresholdHigh);
	ReadSPI(x5b);
	ReadSPI(x6b);
	ReadSPI(highbyte);
	ReadSPI(lowbyte);
    ReadSPI(x1c);
	ReadSPI(0x00);
	ReadSPI(thresholdLow);
	ReadSPI(thresholdHigh);
	ReadSPI(x5c);
	ReadSPI(x6c);
	ReadSPI(highbyte);
	ReadSPI(lowbyte);
    ReadSPI(x1d);
	ReadSPI(0x00);
	ReadSPI(thresholdLow);
	ReadSPI(thresholdHigh);
	ReadSPI(x5d);
	ReadSPI(x6d);
	ReadSPI(highbyte);
	ReadSPI(lowbyte);
	CS=1;
    
}


/* DONE FOR 4 BOARDS :: HAVE TO OUTPUT FLAG*///////////////////////////////////
unsigned short checkFlag(void){
    int flag=0;
    // initialize SPI data
    unsigned short boardType, sensorValue, errorType=192; // errorType 192 (over voltage) corresponds to 11 000000, 
                                                          //which is the first two bits of the error message
    unsigned short errorMsg=0;
// Check SR B //    
	CS=0;
    //Write command for reading SR B
	WriteReadSPI(0x00);
	WriteReadSPI(0x12);
    // Write PEC
	char  x[2] = {0x00,0x12};
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    //Read SR B
	unsigned short x1=0, x2=0, x3=0, x4=0, x5=0, x6=0, xPEC1 =0, xPEC0 = 0;
    unsigned short x1b=0, x2b=0, x3b=0, x4b=0, x5b=0, x6b=0, xPEC1b =0, xPEC0b = 0;
    unsigned short x1c=0, x2c=0, x3c=0, x4c=0, x5c=0, x6c=0, xPEC1c =0, xPEC0c = 0;
    unsigned short x1d=0, x2d=0, x3d=0, x4d=0, x5d=0, x6d=0, xPEC1d =0, xPEC0d = 0;
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
    if ((x3 > 0) ||(x4 > 0) || (x5 > 0)){
        flag = 1;
    }
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    killTime();
    x1b = ReadSPI(0x00);
	x2b = ReadSPI(0x00);
	x3b = ReadSPI(0x00);
    x4b = ReadSPI(0x00);
	x5b = ReadSPI(0x00);
    if ((x3b > 0) ||(x4b > 0) || (x5b > 0)){
        flag = 2;
    }
	x6b = ReadSPI(0x00);
	xPEC1b = ReadSPI(0x00);
	xPEC0b = ReadSPI(0x00);
    x1c = ReadSPI(0x00);
	x2c = ReadSPI(0x00);
	x3c = ReadSPI(0x00);
	x4c = ReadSPI(0x00);
	x5c = ReadSPI(0x00);
    if ((x3c > 0) ||(x4c > 0) || (x5c > 0)){
        flag = 3;
    }
	x6c = ReadSPI(0x00);
	xPEC1c = ReadSPI(0x00);
	xPEC0c = ReadSPI(0x00);
    killTime();
    x1d = ReadSPI(0x00);
	x2d = ReadSPI(0x00);
	x3d = ReadSPI(0x00);
    x4d = ReadSPI(0x00);
	x5d = ReadSPI(0x00);
    if ((x3d > 0) ||(x4d > 0) || (x5d > 0)){
        flag = 4;
    }
	x6d = ReadSPI(0x00);
	xPEC1d = ReadSPI(0x00);
	xPEC0d = ReadSPI(0x00);
	CS=1;
    /* Case statement for determining flag for voltages 1 to 12
    // in each AMS board*/
    switch(flag){
        case 0 :
            break;
        case 1:
            boardType=0; // A corresponds to cells 1-15, so add 0
            if(x3 > 0){
                x3 = x3 >> 3;       
                if(x3 > 0){
                    x3 = x3 >> 2;
                    if(x3 > 2){
                        /* C4 is triggered*/
                        sensorValue = 4 + boardType;
                    }else if(x3 == 0){
                        /*C2 is triggered*/
                        sensorValue = 2 + boardType;
                    }else if(x3 == 1){
                        /*C3 is triggered*/
                        sensorValue = 3 + boardType;
                    }
                }else if(x3 == 0){
                    /* C1 is triggered*/
                    sensorValue = 1 + boardType;
                }
            }else if(x4 > 0){
                x4 = x4 >> 3;       
                if(x4 > 0){
                    x4 = x4 >> 2;
                    if(x4 > 2){
                        /* C8 is triggered*/
                        sensorValue = 8 + boardType;
                    }else if(x4 == 0){
                        /*C6 is triggered*/
                        sensorValue = 6 + boardType;
                    }else if(x4 == 1){
                        /*C7 is triggered*/
                        sensorValue = 7 + boardType;
                    }
                }else if(x4 == 0){
                    /* C5 is triggered*/
                    sensorValue = 5 + boardType;
                }
            }else if(x5 > 0){
                x5 = x5 >> 3;       
                if(x5 > 0){
                    x5 = x5 >> 2;
                    if(x5 > 2){
                        /* C12 is triggered*/
                        sensorValue = 12 + boardType;
                    }else if(x5 == 0){
                        /*C10 is triggered*/
                        sensorValue = 10 + boardType;
                    }else if(x5 == 1){
                        /*C11 is triggered*/
                        sensorValue = 11 + boardType;
                    }
                }else if(x5 == 0){
                    /* C9 is triggered*/
                    sensorValue = 9 + boardType;
                }
            }
            break;
        case 2:
            boardType=15;//B corresponds to 16-30, so add 15
            if(x3b > 0){
                x3b = x3b >> 3;       
                if(x3b > 0){
                    x3b = x3b >> 2;
                    if(x3b > 2){
                        /* C4b is triggered*/
                        sensorValue = 4 + boardType;
                    }else if(x3b == 0){
                        /*C2b is triggered*/
                        sensorValue = 2 + boardType;
                    }else if(x3b == 1){
                        /*C3b is triggered*/
                        sensorValue = 3 + boardType;
                    }
                }else if(x3b == 0){
                    /* C1b is triggered*/
                    sensorValue = 1 + boardType;
                }
            }else if(x4b > 0){
                x4b = x4b >> 3;       
                if(x4b > 0){
                    x4b = x4b >> 2;
                    if(x4b > 2){
                        /* C8b is triggered*/
                        sensorValue = 8 + boardType;
                    }else if(x4b == 0){
                        /*C6b is triggered*/
                        sensorValue = 6 + boardType;
                    }else if(x4b == 1){
                        /*C7b is triggered*/
                        sensorValue = 7 + boardType;
                    }
                }else if(x4b == 0){
                    /* C5b is triggered*/
                    sensorValue = 5 + boardType;
                }
            }else if(x5b > 0){
                x5b = x5b >> 3;       
                if(x5b > 0){
                    x5b = x5b >> 2;
                    if(x5b > 2){
                        /* C12b is triggered*/
                        sensorValue = 12 + boardType;
                    }else if(x5b == 0){
                        /*C10b is triggered*/
                        sensorValue = 10 + boardType;
                    }else if(x5b == 1){
                        /*C11b is triggered*/
                        sensorValue = 11 + boardType;
                    }
                }else if(x5b == 0){
                    /* C9b is triggered*/
                    sensorValue = 9 + boardType;
                }
            }
            break;
        case 3:
            boardType=30;//C corresponds to 31-45, so add 30
            if(x3c > 0){
                x3c = x3c >> 3;       
                if(x3c > 0){
                    x3c = x3c >> 2;
                    if(x3c > 2){
                        /* C4c is triggered*/
                        sensorValue = 4 + boardType;
                    }else if(x3c == 0){
                        /*C2c is triggered*/
                        sensorValue = 2 + boardType;
                    }else if(x3c == 1){
                        /*C3c is triggered*/
                        sensorValue = 3 + boardType;
                    }
                }else if(x3c == 0){
                    /* C1c is triggered*/
                    sensorValue = 1 + boardType;
                }
            }else if(x4c > 0){
                x4c = x4c >> 3;       
                if(x4c > 0){
                    x4c = x4c >> 2;
                    if(x4c > 2){
                        /* C8c is triggered*/
                        sensorValue = 8 + boardType;
                    }else if(x4c == 0){
                        /*C6c is triggered*/
                        sensorValue = 6 + boardType;
                    }else if(x4c == 1){
                        /*C7c is triggered*/
                        sensorValue = 7 + boardType;
                    }
                }else if(x4c == 0){
                    /* C5c is triggered*/
                    sensorValue = 5 + boardType;
                }
            }else if(x5c > 0){
                x5c = x5c >> 3;       
                if(x5c > 0){
                    x5c = x5c >> 2;
                    if(x5c > 2){
                        /* C12c is triggered*/
                        sensorValue = 12 + boardType;
                    }else if(x5c == 0){
                        /*C10c is triggered*/
                        sensorValue = 10 + boardType;
                    }else if(x5c == 1){
                        /*C11c is triggered*/
                        sensorValue = 11 + boardType;
                    }
                }else if(x5c == 0){
                    /* C9c is triggered*/
                    sensorValue = 9 + boardType;
                }
            }
            break;
        case 4:
            boardType=45;//D corresponds to 46-60, so add 45
            if(x3d > 0){
                x3d = x3d >> 3;       
                if(x3d > 0){
                    x3d = x3d >> 2;
                    if(x3d > 2){
                        /* C4 is triggered*/
                        sensorValue = 4 + boardType;
                    }else if(x3d == 0){
                        /*C2 is triggered*/
                        sensorValue = 2 + boardType;
                    }else if(x3d == 1){
                        /*C3 is triggered*/
                        sensorValue = 3 + boardType;
                    }
                }else if(x3d == 0){
                    /* C1 is triggered*/
                    sensorValue = 1 + boardType;
                }
            }else if(x4d > 0){
                x4d = x4d >> 3;       
                if(x4d > 0){
                    x4d = x4d >> 2;
                    if(x4d > 2){
                        /* C8 is triggered*/
                        sensorValue = 8 + boardType;
                    }else if(x4d == 0){
                        /*C6 is triggered*/
                        sensorValue = 6 + boardType;
                    }else if(x4d == 1){
                        /*C7 is triggered*/
                        sensorValue = 7 + boardType;
                    }
                }else if(x4d == 0){
                    /* C5 is triggered*/
                    sensorValue = 5 + boardType;
                }
            }else if(x5d > 0){
                x5d = x5d >> 3;       
                if(x5d > 0){
                    x5d = x5d >> 2;
                    if(x5d > 2){
                        /* C12 is triggered*/
                        sensorValue = 12 + boardType;
                    }else if(x5d == 0){
                        /*C10 is triggered*/
                        sensorValue = 10 + boardType;
                    }else if(x5d == 1){
                        /*C11 is triggered*/
                        sensorValue = 11 + boardType;
                    }
                }else if(x5d == 0){
                    /* C9 is triggered*/
                    sensorValue = 9 + boardType;
                }
            }
            break;            
            
    }
            
// Check Aux Reg D
    CS=0;
    //Write command for reading SR B
	WriteReadSPI(0x00);
	WriteReadSPI(0x0F);
    // Write PEC
	char ox[2] = {0x00,0x0F};
	u = pec15(ox,2);	
	highbyte = (u >> 8);
	lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    //Read SR B
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
    x5 = (0x2F & x5);
    
    /* C13 through C15 flags for AMS board 1*/
    if (x5 > 0){
       flag = 1;
       x5 = x5 >> 3;
       if(x5 == 0){
           /* C13 is triggered */
           sensorValue = 13;
       }else if(x5 > 2){
           /* C15 is triggered */
           sensorValue = 15;
       }else if(x5 == 1){
           /* C14 is triggered*/
           sensorValue = 14;
       }
    }
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1b = ReadSPI(0x00);
	x2b = ReadSPI(0x00);
	x3b = ReadSPI(0x00);
    x4b = ReadSPI(0x00);
	x5b = ReadSPI(0x00);
    x5b = (0x2F & x5b);

    /* C13 through C15 flags for AMS board 2 */
    if (x5b > 0){
       flag = 1;
       x5b = x5b >> 3;
       if(x5b == 0){
           /* C13 is triggered */
           sensorValue = 13+15;
       }else if(x5b > 2){
           /* C15 is triggered */
           sensorValue = 15+15;
       }else if(x5b == 1){
           /* C14 is triggered*/
           sensorValue = 14+15;
       }
    }
	x6b = ReadSPI(0x00);
	xPEC1b = ReadSPI(0x00);
	xPEC0b = ReadSPI(0x00);
	x1c = ReadSPI(0x00);
	x2c = ReadSPI(0x00);
	x3c = ReadSPI(0x00);
	x4c = ReadSPI(0x00);
	x5c = ReadSPI(0x00);
    x5c = (0x2F & x5);

    /* C13 through C15 flags for AMS board 3 */
    if (x5c > 0){
        flag = 1;
       x5c = x5c >> 3;
       if(x5c == 0){
           /* C13 is triggered */
           sensorValue = 13+30;
       }else if(x5c > 2){
           /* C15 is triggered */
           sensorValue = 15+30;
       }else if(x5c == 1){
           /* C14 is triggered*/
           sensorValue = 14+30;
       }
    }
	x6c = ReadSPI(0x00);
	xPEC1c = ReadSPI(0x00);
	xPEC0c = ReadSPI(0x00);
    x1d = ReadSPI(0x00);
	x2d = ReadSPI(0x00);
	x3d = ReadSPI(0x00);
    x4d = ReadSPI(0x00);
	x5d = ReadSPI(0x00);
    x5d = (0x2F & x5b);
    /* C13 through C15 flags for AMS board 4 */
    if (x5d > 0){
        flag = 1;
       x5d = x5d >> 3;
       if(x5d == 0){
           /* C13 is triggered */
           sensorValue = 13+45;
       }else if(x5d > 2){
           /* C15 is triggered */
           sensorValue = 15+45;
       }else if(x5d == 1){
           /* C14 is triggered*/
           sensorValue = 14+45;
       }
    }
	x6d = ReadSPI(0x00);
	xPEC1d = ReadSPI(0x00);
	xPEC0d = ReadSPI(0x00);
    if(flag > 0){
        errorMsg = errorType + sensorValue;
    }
	CS=1;
    return errorMsg;
}
/* DONE FOR 4 BOARDS */////////////////////////////////////////////////////////
void checkVolts(void){
	CS=0;
	WriteReadSPI(0x03);
	WriteReadSPI(0x60);
	// Write PEC
	char  x[2] = {0x03,0x60};
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    WriteReadSPI(0x00);
    u = WriteReadSPI(0x00);
	while(u == 0x00){
        u = WriteReadSPI(0x00);
    }
	// Kill time for 2,537 us (for ADC conversion) + 4.4 ms for REFUP
	//killTimeCV();
    //killTimeCV();
    //killTimeCV();
    CS=1;
	// Read Cell Voltage Register A
	CS=0;
    //Write command
	WriteReadSPI(0x00);
	WriteReadSPI(0x04);
    // Write PEC
	x[0] = 0x00;
    x[1] = 0x04;
	u = pec15(x,2);	
	highbyte = (u >> 8);
	lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    //Read PWM
	unsigned short x1=0, x2=0, x3=0, x4=0, x5=0, x6=0, xPEC1=0,xPEC0=0;
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
	CS=1;
}

/* DONE FOR 4 BOARDS EXCEPT FOR RETURN *///////////////////////////////////////

void checkSumOfVolts(void){
	CS=0;
	WriteReadSPI(0x05);
	WriteReadSPI(0x68);
	// Write PEC
	char  x[2] = {0x05,0x68};
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    WriteReadSPI(0x00);
    WriteReadSPI(0x00);
    WriteReadSPI(0x00);
    WriteReadSPI(0x00);
    u = WriteReadSPI(0x00);
	while(u == 0x00){
        u = WriteReadSPI(0x00);
    }
    CS=1;
    
	// Read Cell Voltage Register A
	CS=0;
    //Write command
	WriteReadSPI(0x00);
	WriteReadSPI(0x10);
    // Write PEC
	x[0] = 0x00;
    x[1] = 0x10;
	u = pec15(x,2);	
	highbyte = (u >> 8);
	lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    //Read Sum of Volts -- each board writes out 8 bytes
	unsigned short x1=0, x2=0, x3=0, x4=0, x5=0, x6=0, xPEC1=0,xPEC0=0,sum1=0;
    unsigned short sum2=0, sum3=0, sum4=0;
    unsigned short sumAll = 0;
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    sum1 = ((x2 << 8) & 0xFF00);
    sum1 = sum1 + x1;
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    sum2 = (x2 << 8) & 0xFF00;
    sum2 = sum2 + x1;
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    sum3 = (x2 << 8) & 0xFF00;
    sum3 = sum3 + x1;
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
    x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    sum4 = (x2 << 8)& 0xFF00;
    sum4 = sum4 + x1;
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
	CS=1;
    sumAll = (sum4 + sum1 + sum2 + sum3); // Send sumAll to the master
    endSum = (((float) sumAll)*100*30/1000000);
    //LCD_setpos(0,0);
    //printf("Battery pack is at %3.2f Volts.");
    sumAll=0;

    
    /* When printing this, cast as float and divide by 10^6
    // which is the actual value of the sum of cells. 
    // for simplicity, use %3.2f in printf statement */
    
}
/* Read Aux will read the given register, and compute the avg
// temperature of the three sensors. Then it will add to a
// recurring count of the averages. currentTemp should be
// zeroed before calling ReadAux. After ReadAux has been called
// for all registers, currentTemp will be the sum of the four
 / average temperatures, so it must be divided by 4 to get the
 / true average. */
void ReadAux(unsigned short cmd0, unsigned short cmd1, char* registerAux, int regCount){
    // Read Aux Register A :: cmd0 = 0x00 cmd1 = 0x0C
	CS=0;
    //Write command
	WriteReadSPI(cmd0);
	WriteReadSPI(cmd1);
    // Write PEC
    char  x[2];
	x[0] = cmd0;
    x[1] = cmd1;
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    // Need to put u into the SPI
    //Read Sum of Volts -- each board writes out 8 bytes
	unsigned short x1=0, x2=0, x3=0, x4=0, x5=0, x6=0, xPEC1=0,xPEC0=0;
    unsigned long G1V, G2V, G3V, G1Vb, G2Vb, G3Vb, G1Vc, G2Vc, G3Vc, G1Vd, G2Vd, G3Vd;
    unsigned long threshold;
/* First board*/
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    G1V = (x2 << 8);
    G1V = G1V + x1;
    if(G1V > threshold){
        /* printf("Temperature sensor 1+regCount on board 1 in Aux Reg %c is above threshold", registerAux);*/
    }
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
    if(registerAux == "D"){
        x3=0;
        x4=0;
    }
    G2V = (x4 << 8);
    G2V = G2V + x3;
    if(G2V > threshold){
        /* printf("Temperature sensor 2+regCount in Board 1 is above threshold", registerAux);*/
    }
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
    if(registerAux == "B" || registerAux == "D"){
        x6=0;
        x5=0;
    }
    G3V = (x6 << 8);
    G3V = G3V + x5;
    if(G3V > threshold){
        /* printf("Temperature sensor 3+regCount in Board 1 is above threshold", registerAux);*/
    }
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
/* Second board*/
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    G1Vb = (x2 << 8);
    G1Vb = G1Vb + x1;
    if(G1Vb > threshold){
        /* printf("Temperature sensor 1+regCount in Board 2 is above threshold", registerAux);*/
    }
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
    if(registerAux == "D"){
        x3=0;
        x4=0;
    }
    G2Vb = (x4 << 8);
    G2Vb = G2Vb + x3;
    if(G2Vb > threshold){
        /* printf("Temperature sensor 2+regCount in Board 2 is above threshold", registerAux);*/
    }
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
    if(registerAux == "B" || registerAux == "D"){
        x6=0;
        x5=0;
    }
    G3Vb = (x6 << 8);
    G3Vb = G3Vb + x5;
    if(G3Vb > threshold){
        /* printf("Temperature sensor 3+regCount in Board 3 is above threshold", registerAux);*/
    }
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
/* Third board*/
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    G1Vc = (x2 << 8);
    G1Vc = G1Vc + x1;
    if(G1Vc > threshold){
        /* printf("Temperature sensor 1+regCount in Board 3 is above threshold", registerAux);*/
    }
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
    if(registerAux == "D"){
        x3=0;
        x4=0;
    }
    G2Vc = (x4 << 8);
    G2Vc = G2Vc + x3;
    if(G2Vc > threshold){
        /* printf("Temperature sensor 2+regCount in Board 3 is above threshold", registerAux);*/
    }
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
    if((registerAux == "B") || (registerAux == "D")){
        x6=0;
        x5=0;
    }
    G3Vc = (x6 << 8);
    G3Vc = G3Vc + x5;
    if(G3Vc > threshold){
        /* printf("Temperature sensor 3+regCount in Board 3 is above threshold", registerAux);*/
    }
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
/* Fourth board*/
	x1 = ReadSPI(0x00);
	x2 = ReadSPI(0x00);
    G1Vd = (x2 << 8);
    G1Vd = G1Vd + x1;
    if(G1Vd > threshold){
        /* printf("Temperature sensor 1+regCount in Board 4 is above threshold", registerAux);*/
    }
	x3 = ReadSPI(0x00);
	x4 = ReadSPI(0x00);
    if(registerAux == "D"){
        x3=0;
        x4=0;
    }
    G2Vd = (x4 << 8);
    G2Vd = G2Vd + x3;
    if(G2Vd > threshold){
        /* printf("Temperature sensor 2+regCount in Board 4 is above threshold", registerAux);*/
    }
	x5 = ReadSPI(0x00);
	x6 = ReadSPI(0x00);
    if(registerAux == "B" || registerAux == "D"){
        x6=0;
        x5=0;
    }
    G3Vd = (x6 << 8);
    G3Vd = G3Vd + x5;
    if(G3Vd > threshold){
        /* printf("Temperature sensor 3+regCount in Board 4 is above threshold", registerAux);*/
    }
	xPEC1 = ReadSPI(0x00);
	xPEC0 = ReadSPI(0x00);
	CS=1;
    currentTemp = currentTemp + (G1V+G1Vb+G1Vc+G1Vd+G2V+G2Vb+G2Vc+G2Vd+G3V+G3Vb+G3Vc+G3Vd)/16;
    
    
}
void readTemperatures(void){
	CS=0;
	WriteReadSPI(0x05);
	WriteReadSPI(0x60);
	// Write PEC 0b0
	char  x[2] = {0x05,0x60};
	unsigned short u = pec15(x,2);	
	unsigned short highbyte = (u >> 8);
	unsigned short lowbyte = (u & 0x00FF);
	WriteReadSPI(highbyte);
	WriteReadSPI(lowbyte);
    WriteReadSPI(0x00);
    u = WriteReadSPI(0x00);
	while(u == 0x00){
        u = WriteReadSPI(0x00);
    }
    CS=1;
    currentTemp = 0;
    /* Read A */
    ReadAux(0x00, 0x0C, "A", 0);
    /* Read B */
    ReadAux(0x00, 0x0E, "B", 3);
    /* Read C */
    ReadAux(0x00, 0x0D, "C", 5);
    /* Read D */
    ReadAux(0x00, 0x0F, "D", 8);
    savedTemp = currentTemp/4; 
    currentTemp=0;
}
/////////////// TWO CALLS OF THIS WAKES UP ONE BOARD //////////////////////////
void WakeUpAMS(void){
    // Set differential activity by keeping clock high on idle and CS low
    CS = 0;
    // Wait for t-dwell = 240 ns
    killTime();
    // Wait for t-wake. HAS TO BE 400 us at least
    killTime();
    killTime();
    killTime();
    killTime();
    killTime();
    killTime();
    killTime();
    CS=1;
}
///////////////////////////////////////////////////////////////////////////////
void killTime(void){
    int x=0;
    // Wait for 60 us
    while(x<84){
        x=x+1;
    }
}
void initSPI(void){
    CS = 1;                     // Set CS high (idle state)
    
    AD1PCFG = 0xFFFF;           // set all pins to digital
    DDPCON = 0x0000;            // disable JTAG
    IEC1bits.SPI2EIE = 0;       // SPI interrupts disabled
    IEC1bits.SPI2RXIE = 0;
    IEC1bits.SPI2TXIE = 0;
    SPI2CONbits.ON = 0;         // Turn off SPI module
     
    SPI2BUF = 0;                // Clear the receive buffer
     
    SPI2BRG = 19;                // FSCK = 1MHz
     //SPI4BRG = 0;                // FSCK = 20MHz (once we get it working)
     
    SPI2STATbits.SPIROV = 0;    // Clear overflow flag
     
     
    /* SPI1CON settings */
    SPI2CONbits.FRMEN = 0;      // Framed SPI support is disabled
    SPI2CONbits.SIDL = 0;       // Continue operation in IDLE mode
    SPI2CONbits.DISSDO = 0;     // SDO1 pin is controlled by the module
    //SPI2CONbits.MODE16 = 0;     // 16 bit mode
    //SPI2CONbits.MODE32 = 0;
    SPI2CONbits.CKP = 1;        // Idle state for clock is high, active state is low, on rising edge
    SPI2CONbits.CKE = 0; //changed it from 0       // Output data changes on transition from idle to active
    SPI2CONbits.SSEN = 0;       // Not in slave mode
    SPI2CONbits.MSTEN = 1;      // Master mode
    SPI2CONbits.SMP = 0;        // Input data sampled at the middle of data output time
     
    SPI2CONbits.ON = 1;         // Turn module on
}

short WriteReadSPI(unsigned short i){
    SPI2BUF = i;                    // Write to buffer for transmission
    while (!SPI2STATbits.SPIRBF);   // Wait for transfer to be completed
    return SPI2BUF;                 // Return the received value
}

short ReadSPI(unsigned short i){
    SPI2BUF = i;                    // Write to buffer for transmission
    while (!SPI2STATbits.SPIRBF);   // Wait for transfer to be completed
    //unsigned short x = SPI2BUF;
    return SPI2BUF;                 // Return the received value
}
/* This function will...
 /// :: Write the overvoltage value to the appropriate registers
 /// :: Check the individual voltage of cells (necessary to trigger overvoltage flag 
 /// :: Check to see the if the flag is triggered and output either 1 or 0 to a variable*/

unsigned short xFlag=0;
void OvervoltageCode(void){
    LED=0;
    checkVolts();
    xFlag = checkFlag();
}

void WakeUp4Boards(void){
    /* WakeUp AMS has to be called twice per daisy-chained board*/
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
    WakeUpAMS();
}
int main(int argc, char** argv){
    //LCD_init();
    //LCD_clear();
    initSPI();
    init_PEC15_Table();
    int flagBit=0;
    TRISDbits.TRISD7 = 0;
                      // initialize SPI module
    WakeUp4Boards();
    writeOV();                  // Write Overvoltage values once
    while(1){
        checkSumOfVolts();
        //OvervoltageCode();
        /* Send xFlag and sumAll, both unsigned shorts  */
        
        //readTemperatures();
    }
    
    
    return (EXIT_SUCCESS);
}

